@@ -1,10 +1,9 @@
-#!/usr/bin/env perl
+#!/usr/bin/perl
 
 # Simple socks5 client
 # gets google.com main page
 # implemented with IO::Socket::Socks
 
-use lib '../lib';
 use strict;
 use IO::Socket::Socks;
 
@@ -12,7 +11,7 @@ use IO::Socket::Socks;
 #$IO::Socket::Socks::SOCKS5_RESOLVE = 0;
 
 my $socks = new IO::Socket::Socks(ProxyAddr=>"127.0.0.1",
-                                  ProxyPort=>"1080",
+                                  ProxyPort=>"1090",
                                   ConnectAddr=>"www.google.com",
                                   ConnectPort=>80,
                                   # uncomment lines below if you want to use authentication
